from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import RandomizedSearchCV, train_test_split
from mlxtend.plotting import plot_decision_regions
import matplotlib.pyplot as plt
import numpy as np
import warnings
warnings.simplefilter('ignore')

def logreg(x, y):
    xtrain, xtest, ytrain, ytest = train_test_split(x, y, test_size=0.3, random_state=4
                                                   ,shuffle=True)
    parameters={'penalty':['l1','l2','elasticnet', 'none'],
                'C':[np.arange(0.1,10,0.1),1,10,100,1000],
                'random_state':range(1,50),
                'solver':['newton-cg', 'lbfgs', 'liblinear', 'sag', 'saga'],
                'max_iter':[1,10,100]}

    main_model = LogisticRegression()
    scv_model = RandomizedSearchCV(main_model, parameters, cv=10)
    best_params = np.array([])
    best_score = np.array([])
    for i in range(x.ndim*3):
        scv_model.fit(xtrain, ytrain)
        best_params = np.append(best_params, scv_model.best_params_)
        best_score = np.append(best_score, scv_model.best_score_)

    best_index = np.argmax(best_score)
    print('The best accuracy depends on you dataset is {}%'
          .format(round(best_score[best_index]*100, 2)))
    print('The parameters are: ')
    print(best_params[best_index])
    
    def plotting(xtrain,ytrain):
        plt.title('LogisticRgression Decision Region Boundary')
        plot_decision_regions(xtrain, ytrain, clf=scv_model, zoom_factor=2)
        plt.show()
    plotting(xtrain, ytrain)



from sklearn.cluster import DBSCAN
from sklearn.neighbors import NearestNeighbors
from sklearn import metrics
import numpy as np

def dbscan(x):
    NN=NearestNeighbors(n_neighbors=5).fit(x)
    distance, indice = NN.kneighbors(x)
    eps = distance.max()
    min_samples = x.ndim*2
    silhouette = np.array([])
    model = DBSCAN(eps=eps, min_samples=min_samples).fit(x)
    label = model.labels_
    silhouette=np.append(silhouette, metrics.silhouette_score(x, label))

    if silhouette < 0:
        print('''
        The score of clustring is {}%
        Datapoints have been assigned to the wrong clusters.
        You should consider preprocessing and dataset cleaning.
        and the parameters are:
        '''.format(round(silhouette[0]*100, 2)))
        print('epsilon: ',eps)
        print('number of samples: ',min_samples)
    elif silhouette >= 0 and silhouette < 0.1:
        print('''
        The score of clustring is {}%
        ""overlapping clusters""
        You should consider preprocessing.
        and the parameters are:
        '''.format(round(silhouette[0]*100, 2)))
        print('epsilon: ',eps)
        print('Optimal number of samples: ',min_samples)
    elif silhouette > 0.1 and silhouette < 0.5:
        print('''
        The score of clustring is {}%.
        Datapoints has been approximately assigned to the right clusters.
        and the parameters are:
        '''.format(round(silhouette[0]*100, 2)))
        print('Best epsilon: ',eps)
        print('Optimal number of samples: ',min_samples)
    elif silhouette >= 0.5:
        print('''
        The score of clustring is {}%.
        Good clustering.
        and the parameters are:
        '''.format(round(silhouette[0]*100, 2)))
        print('Best epsilon: ',eps)
        print('Optimal number of samples: ',min_samples)



from sklearn.svm import SVC
from sklearn.model_selection import RandomizedSearchCV, train_test_split
from sklearn.preprocessing import StandardScaler
import numpy as np
import warnings
warnings.simplefilter("ignore")

def svc(x,y):
    xtrain, xtest, ytrain, ytest = train_test_split(x, y, test_size=0.3, random_state=1
                                                   ,shuffle=True)
    ss=StandardScaler()
    xtrain=ss.fit_transform(xtrain)
    xtest=ss.fit_transform(xtest)
    parameters={'kernel':['rbf','sigmoid','poly','linear'],
                'C':[np.arange(1,10,1),100,1000,1e5],
                'degree':np.arange(0,11,1),
                'gamma':['scale','auto',0.1, 1, 10, 100]}
    main_model = SVC()
    scv_model = RandomizedSearchCV(main_model, parameters, cv=10)
    best_params = np.array([])
    best_score = np.array([])
    for i in range(x.ndim*2):
        scv_model.fit(xtrain, ytrain)
        best_params = np.append(best_params, scv_model.best_params_)
        best_score = np.append(best_score, scv_model.best_score_)

    best_index = np.argmax(best_score)
    print('The best accuracy depends on you dataset is {}%'
          .format(round(best_score[best_index]*100,2)))
    print('The parameters are: ')
    print(best_params[best_index])



from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import RandomizedSearchCV, train_test_split
from mlxtend.plotting import plot_decision_regions
import matplotlib.pyplot as plt
import numpy as np

def decisiontree(x,y):
    xtrain, xtest, ytrain, ytest = train_test_split(x, y, test_size=0.3, random_state=4
                                               ,shuffle=True)
    parameters ={'max_depth':range(1,10,1),
                'random_state':range(1,50,1),
                'max_features':['auto', 'sqrt', 'log2'],
                'splitter':['best', 'random'],
                'criterion':['gini', 'entropy']}

    main_model = DecisionTreeClassifier()
    scv_model = RandomizedSearchCV(main_model, parameters, cv=5)
    best_params = np.array([])
    best_score = np.array([])
    for i in range(x.ndim*3):
        scv_model.fit(xtrain, ytrain)
        best_params = np.append(best_params, scv_model.best_params_)
        best_score = np.append(best_score, scv_model.best_score_)

    best_index = np.argmax(best_score)
    print('The best accuracy depends on you dataset is {}%'
          .format(round(best_score[best_index]*100,2)))
    print('The parameters are: ')
    print(best_params[best_index])
    
    def plotting(xtrain, ytrain):
        plt.title('DecisionTree Decision Region Boundary')
        plot_decision_regions(xtrain, ytrain, clf=scv_model, zoom_factor=2.0)
        plt.show()
    plotting(xtrain, ytrain)



import numpy as np

def root_mean_squared_error(y_true, y_pred):
    rmse = np.sqrt(np.square(np.subtract(y_true, y_pred)).mean())
    return rmse
    
def root_mean_squared_log_error(y_true, y_pred):
    rmsle = np.sqrt(np.square(np.subtract(np.log(y_pred + 1), np.log(y_true + 1))).mean())
    return rmsle

def root_mean_squared_precentage_error(y_true, y_pred):
    rmspe = (np.sqrt(np.mean(np.square(np.subtract(y_true, y_pred) / y_true)))) * 100
    return rmspe

def symmetric_mean_absolute_precentage_error(y_true, y_pred):
    smape = 100/len(y_true) * np.sum(2 * np.abs(np.subtract(y_pred, y_true)) / (np.abs(y_true) + np.abs(y_pred)))
    return smape
    
def mean_bias_error(y_true, y_pred):
    mbe = np.mean(np.subtract(y_pred, y_true))
    return mbe

def relative_squared_error(y_true, y_pred): 
    rse = np.sum(np.square(np.subtract(y_true, y_pred))) / np.sum(np.square(np.subtract(np.average(y_true), y_true)))
    return rse

def root_relative_squared_error(y_true, y_pred):
    rrse = np.sqrt(np.sum(np.square(np.subtract(y_true, y_pred))) / np.sum(np.square(np.subtract(np.average(y_true), y_true))))
    return rrse
    
def relative_absolute_error(y_true, y_pred):
    rae = np.sum(np.abs(np.subtract(y_true, y_pred))) / (np.sum(np.abs(np.subtract(y_true, np.average(y_true)))) + 1e-10)
    return rae

def median_absolute_percentage_error(y_true, y_pred): #??????
    mape = np.median((np.abs(np.subtract(y_true, y_pred)/ y_true)) ) * 100
    return mape

def mean_absolute_percentage_error(y_true, y_pred): #??????
    mape = np.mean((np.abs(np.subtract(y_true, y_pred)/ y_true))) * 100
    return mape


from sklearn.neighbors import KNeighborsClassifier
from sklearn.model_selection import RandomizedSearchCV, train_test_split
from mlxtend.plotting import plot_decision_regions
import matplotlib.pyplot as plt
import numpy as np

def knn(x,y):
    xtrain, xtest, ytrain, ytest = train_test_split(x, y, test_size=0.3,random_state=2
                                                   ,shuffle=True)
    parameters={'n_neighbors':range(1,20),
                'weights':['uniform', 'distance'],
                'algorithm':['auto', 'ball_tree', 'kd_tree', 'brute'],
                'metric':['minkowski','manhattan','euclidean']}
    main_model = KNeighborsClassifier()
    scv_model = RandomizedSearchCV(main_model, parameters, cv=10)
    best_params = np.array([])
    best_score = np.array([])
    for i in range(x.ndim*3):
        scv_model.fit(xtrain, ytrain)
        best_params = np.append(best_params, scv_model.best_params_)
        best_score = np.append(best_score, scv_model.best_score_)
    best_index = np.argmax(best_score)
    print('The best accuracy depends on you dataset is {}%'
          .format(round(best_score[best_index]*100, 2)))
    print('The parameters are: ')
    print(best_params[best_index])
    
    def plotting(xtrain,ytrain):
        plt.title('KNN Decision Region Boundary')
        plot_decision_regions(xtrain, ytrain, clf=scv_model, legend='best'
                              ,zoom_factor=2)
        plt.show()
    plotting(xtrain, ytrain)



from .dbscan import dbscan
from .decisiontree import decisiontree
from .kmeans import kmeans
from .knn import knn
from .logreg import logreg
from .meanshift import meanshift
from .svc import svc
from .metrics import root_mean_squared_error
from .metrics import root_mean_squared_log_error
from .metrics import mean_bias_error
from .metrics import relative_squared_error
from .metrics import relative_absolute_error



from sklearn.cluster import KMeans
from sklearn import metrics
import numpy as np
import matplotlib.pyplot as plt

def kmeans(x):
    calinski_harabasz_score = np.array([])
    for i in range(2,20):
        model = KMeans(n_clusters=i).fit(x)
        label = model.labels_
        calinski_harabasz_score = np.append(calinski_harabasz_score
                                            ,metrics.calinski_harabasz_score(x, label))
    best_score = np.argmax(calinski_harabasz_score)
    best_proba = calinski_harabasz_score[best_score]
    plt.style.use("fivethirtyeight")
    plt.figure(dpi=80, figsize=(9,5))
    plt.title('Model evaluation result')
    plt.grid()
    plt.margins(0.05)
    plt.xlabel('Number of clusters', fontsize=12, color='red')
    plt.ylabel('Score of evaluation', fontsize=12, color='red')
    plt.xticks(range(2, 20))
    plt.yticks()
    plt.annotate('Optimal clustering', xytext=(best_score+2, best_proba)
                 ,xy=(best_score+2, best_proba), fontsize=9, arrowprops={'color':'violet'})
    plt.plot(range(2,20),calinski_harabasz_score,ls='--',marker='x',lw=2,label='calinski score')
    plt.grid()
    plt.legend(loc='best')
    plt.show()



import setuptools
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.txt").read_text()

setuptools.setup(
    author="MohammadReza Barghi",
    author_email="info@genesiscube.ir",
    name='autooptimizer',
    license="MIT",
    description='AutoOptimizer is a python package for optimize ML algorithms.',
    version='v0.4.1',
    long_description= long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/mrb987/autooptimizer',
    packages=setuptools.find_packages(),
    python_requires=">=3",
    install_requires=['sklearn',
                      'numpy',
                      'matplotlib',
                      'mlxtend',],
    keyword=['python', 'machine learning', 'sklearn', 'data science','regression metrics'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',],
)



from sklearn.cluster import MeanShift, estimate_bandwidth
from sklearn import metrics
import numpy as np

def meanshift(x):
    bandwidth = estimate_bandwidth(x,quantile=0.3, random_state=2)
    model = MeanShift(bandwidth=bandwidth).fit(x)
    label = model.labels_
    silhouette = metrics.silhouette_score(x, label)

    if silhouette < 0:
        print('''
        The score of clustring is {}%
        Datapoints have been assigned to the wrong clusters.
        You should consider preprocessing and dataset cleaning.
        and the parameters are:
        '''.format(round(silhouette*100, 2)))    
        print('the assigned bandwidth is: ', bandwidth)
    elif silhouette >= 0 and silhouette < 0.1:
        print('''
        The score of clustring is {}%.
        ""Overlapping clusters""
        You should consider preprocessing.
        and the parameters are:
        '''.format(round(silhouette*100, 2)))
        print('the assigned bandwidth is: ', bandwidth)
    elif silhouette > 0.1 and silhouette <= 0.5:
        print('''
        The score of clustring is {}%.
        Datapoints have been approximately assigned to the right clusters
        and the parameters are:
        '''.format(round(silhouette*100, 2)))
        print('the assigned bandwidth is: ', bandwidth)
    elif silhouette > 0.5:
        print('''
        The score of clustring is {}%.
        Good clustering
        and the parameters are:
        '''.format(round(silhouette*100, 2)))
        print('The assigned bandwidth is: ', bandwidth)
